package com.mobileconnected.amap.map;

import com.amap.api.maps.AMap;
import com.amap.api.maps.model.Circle;
import com.amap.api.maps.model.CircleOptions;

import io.flutter.plugin.common.MethodChannel;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CirclesController {
    private final Map<String, CircleController> circleIdToController;
    private final Map<String, String> googleMapsCircleIdToDartCircleId;
    private final MethodChannel methodChannel;
    private AMap aMap;

    CirclesController(MethodChannel methodChannel) {
        this.circleIdToController = new HashMap<>();
        this.googleMapsCircleIdToDartCircleId = new HashMap<>();
        this.methodChannel = methodChannel;
    }

    void setAmapMap(AMap aMap) {
        this.aMap = aMap;
    }

    void addCircles(List<Object> circlesToAdd) {
        if (circlesToAdd != null) {
            for (Object circleToAdd : circlesToAdd) {
                addCircle(circleToAdd);
            }
        }
    }

    void changeCircles(List<Object> circlesToChange) {
        if (circlesToChange != null) {
            for (Object circleToChange : circlesToChange) {
                changeCircle(circleToChange);
            }
        }
    }

    void removeCircles(List<Object> circleIdsToRemove) {
        if (circleIdsToRemove == null) {
            return;
        }
        for (Object rawCircleId : circleIdsToRemove) {
            if (rawCircleId == null) {
                continue;
            }
            String circleId = (String) rawCircleId;
            final CircleController circleController = circleIdToController.remove(circleId);
            if (circleController != null) {
                circleController.remove();
                googleMapsCircleIdToDartCircleId.remove(circleController.getGoogleMapsCircleId());
            }
        }
    }

    boolean onCircleTap(String googleCircleId) {
        String circleId = googleMapsCircleIdToDartCircleId.get(googleCircleId);
        if (circleId == null) {
            return false;
        }
        methodChannel.invokeMethod("circle#onTap", Convert.circleIdToJson(circleId));
        CircleController circleController = circleIdToController.get(circleId);
        if (circleController != null) {
            return circleController.consumeTapEvents();
        }
        return false;
    }

    private void addCircle(Object circle) {
        if (circle == null) {
            return;
        }
        CircleBuilder circleBuilder = new CircleBuilder();
        String circleId = Convert.interpretCircleOptions(circle, circleBuilder);
        CircleOptions options = circleBuilder.build();
        addCircle(circleId, options, circleBuilder.consumeTapEvents());
    }

    private void addCircle(String circleId, CircleOptions circleOptions, boolean consumeTapEvents) {
        final Circle circle = aMap.addCircle(circleOptions);
        CircleController controller = new CircleController(circle, consumeTapEvents);
        circleIdToController.put(circleId, controller);
        googleMapsCircleIdToDartCircleId.put(circle.getId(), circleId);
    }

    private void changeCircle(Object circle) {
        if (circle == null) {
            return;
        }
        String circleId = getCircleId(circle);
        CircleController circleController = circleIdToController.get(circleId);
        if (circleController != null) {
            Convert.interpretCircleOptions(circle, circleController);
        }
    }

    @SuppressWarnings("unchecked")
    private static String getCircleId(Object circle) {
        Map<String, Object> circleMap = (Map<String, Object>) circle;
        return (String) circleMap.get("circleId");
    }
}
