//
//  MAMapView+CenterAroundVisiableOverlays.m
//  nb_map
//
//  Created by Guo Longxiang, (Longxiang.Guo@partner.bmw.com) on 2019/11/6.
//

#import "MAMapView+CenterAroundVisiableOverlays.h"
#import <objc/runtime.h>
#import <objc/objc.h>
#import <UIKit/UIKit.h>
#import "NBCommonUtility.h"

@implementation MAMapView(CenterAroundVisiableOverlays)

static NSString* NB_ANNOTATION_OVERLAYS_KEY = @"nb_visibale_annotation_overlays_key";
static NSString* NB_POLYLINE_OVERLAYS_KEY = @"nb_visibale_polyline_overlays_key";
static NSString* NB_POLYGON_OVERLAYS_KEY = @"nb_visibale_polyon_overlays_key";
static NSString* NB_CIRCLE_OVERLAYS_KEY = @"nb_visibale_circles_overlays_key";
static NSString* NB_SHOULD_CENTER_VISIABLE_OVERLAYS_KEY = @"nb_should_center_visiable_overlays_key";

- (void)centerWithRule:(NBCenterMapRule)rules padding:(UIEdgeInsets)padding animated:(BOOL)animated {
    if (![self shouldCenterVisiableOverlays]){ return; }
    MAMapRect markersRect = MAMapRectZero, polylinesRect, polygonsRect;
    if ((rules & NBCenterMapAroundPins) && [self visiableMarkers].count > 0) {
        markersRect = [NBCommonUtility minMapRectForAnnotations:[self visiableMarkers]];
        if (markersRect.size.width == 0 && markersRect.size.height == 0) {
            MAMapPoint mapPoint = MAMapPointForCoordinate([self visiableMarkers].firstObject.coordinate);
            double x = mapPoint.x - 100.0;
            double y = mapPoint.y - 200.0;
            markersRect = MAMapRectMake(x, y, 100.0, 200.0);
        }
    }
    if ((rules & NBCenterMapAroundPolylines) && [self visiablePolylines].count > 0) {
        polylinesRect = [NBCommonUtility mapRectForOverlays:[self visiablePolylines]];
        markersRect = MAMapRectUnion(markersRect, polylinesRect);
    }
    if ((rules & NBCenterMapAroundPolygons) && [self visiablePolygons].count > 0) {
        polygonsRect = [NBCommonUtility mapRectForOverlays:[self visiablePolygons]];
        markersRect = MAMapRectUnion(markersRect, polygonsRect);
    }
    NSUInteger circleCount = [self visiableCircles].count;
    if (rules & NBCenterMapAroundCircles) {
        for (NSUInteger index = 0; index < circleCount; ++index) {
            NBCircle* circle = [self visiableCircles][index];
            markersRect = MAMapRectUnion(markersRect, circle.boundingMapRect);
        }
    }
    NSLog(@"\ncenterWithRule: x: %lf  y: %lf  w: %lf  h: %lf", markersRect.origin.x, markersRect.origin.y, markersRect.size.width, markersRect.size.height);
    [self setVisibleMapRect:markersRect edgePadding:padding animated:animated];
    [self setShouldCenterVisiableOverlays:false];
}
- (void)updateCachedMarker:(NBMarker*)marker event: (NBVisiableOverlayEvent)event{
    switch (event) {
        case NBVisiableOverlayAdd:
            NSLog(@"updateCachedMarker add: %lf, %lf, icon: %@", marker.coordinate.latitude, marker.coordinate.longitude, marker.icon);
            if ([[self visiableMarkers]containsObject:marker]) { return; }
            [[self visiableMarkers]addObject:marker]; [self setShouldCenterVisiableOverlays:true];
            break;
        case NBVisiableOverlayDelete:
            if (![[self visiableMarkers]containsObject:marker]) { return; }
            [[self visiableMarkers]removeObject:marker]; [self setShouldCenterVisiableOverlays:true];
            NSLog(@"updateCachedMarker delete: %lf, %lf, icon: %@", marker.coordinate.latitude, marker.coordinate.longitude, marker.icon);
            break;
    }
}
- (void)updateCachedOverlay:(id<MAOverlay>)overlay event: (NBVisiableOverlayEvent)event{
    
    switch (event) {
        case NBVisiableOverlayAdd:
            if ([overlay isKindOfClass:[NBPolyline class]]) {
                NBPolyline* polyine = (NBPolyline*)overlay;
                if ([[self visiablePolylines]containsObject:overlay]) { return; }
                [[self visiablePolylines]addObject:polyine];
                [self setShouldCenterVisiableOverlays:true];
            } else if ([overlay isKindOfClass:[NBPolygon class]]) {
                NBPolygon* polygon = (NBPolygon*)overlay;
                if ([[self visiablePolygons]containsObject:overlay]) { return; }
                [[self visiablePolygons]addObject:polygon];
                [self setShouldCenterVisiableOverlays:true];
            } else if ([overlay isKindOfClass:[NBCircle class]]) {
                NBCircle* circle = (NBCircle*)overlay;
                if ([[self visiableCircles]containsObject:circle]) { return; }
                [[self visiableCircles] addObject:circle];
                [self setShouldCenterVisiableOverlays:true];
            } else {
                [self raiseExpectionWithOverlay:overlay];
            }
            break;
        case NBVisiableOverlayDelete:
            if ([overlay isKindOfClass:[NBPolyline class]]) {
                NBPolyline* polyine = (NBPolyline*)overlay;
                if (![[self visiablePolylines]containsObject:overlay]) { return; }
                [[self visiablePolylines]removeObject:polyine];
                [self setShouldCenterVisiableOverlays:true];
            } else if ([overlay isKindOfClass:[NBPolygon class]]) {
                NBPolygon* polygon = (NBPolygon*)overlay;
                if (![[self visiablePolygons]containsObject:overlay]) { return; }
                [[self visiablePolygons]removeObject:polygon];
                [self setShouldCenterVisiableOverlays:true];
            } else if ([overlay isKindOfClass:[NBCircle class]]) {
                NBCircle* circle = (NBCircle*)overlay;
                if (![[self visiableCircles]containsObject:overlay]) { return; }
                [[self visiableCircles] removeObject:circle];
                [self setShouldCenterVisiableOverlays:true];
            } else {
                [self raiseExpectionWithOverlay:overlay];
            }
            break;
    }
}

- (void)setVisiableMarkers:(NSArray<NBMarker*>*)visiableMarkers {
    objc_setAssociatedObject(self, &NB_ANNOTATION_OVERLAYS_KEY, visiableMarkers, OBJC_ASSOCIATION_RETAIN);
}
- (NSMutableArray<NBMarker*>*)visiableMarkers {
    NSMutableArray<NBMarker*>* visiableMarkers = objc_getAssociatedObject(self, &NB_ANNOTATION_OVERLAYS_KEY);
    if(visiableMarkers == nil) {
        visiableMarkers = [NSMutableArray arrayWithCapacity:0];
        [self setVisiableMarkers:visiableMarkers];
    }
    return visiableMarkers;
}
- (void)setVisiablePolylines:(NSArray<NBPolyline*>*)visbalePolylines{
    objc_setAssociatedObject(self, &NB_POLYLINE_OVERLAYS_KEY, visbalePolylines, OBJC_ASSOCIATION_RETAIN);
}
- (NSMutableArray<NBPolyline*>*)visiablePolylines {
    NSMutableArray<NBPolyline*>* visibalePolylines = objc_getAssociatedObject(self, &NB_POLYLINE_OVERLAYS_KEY);
    if(visibalePolylines == nil) {
        visibalePolylines = [NSMutableArray arrayWithCapacity:0];
        [self setVisiablePolylines:visibalePolylines];
    }
    return visibalePolylines;
}
- (void)setVisibalePolygons:(NSArray<NBPolygon*>*)visiablePolygons{
    objc_setAssociatedObject(self, &NB_POLYGON_OVERLAYS_KEY, visiablePolygons, OBJC_ASSOCIATION_RETAIN);
}
- (NSMutableArray<NBPolygon*>*)visiablePolygons{
    NSMutableArray<NBPolygon*>* visiablePolygons = objc_getAssociatedObject(self, &NB_POLYGON_OVERLAYS_KEY);
    if(visiablePolygons == nil) {
        visiablePolygons = [NSMutableArray arrayWithCapacity:0];
        [self setVisibalePolygons:visiablePolygons];
    }
    return visiablePolygons;
}
- (void)setVisiableCircles:(NSArray<NBCircle*>*)visiableCircles{
    objc_setAssociatedObject(self, &NB_CIRCLE_OVERLAYS_KEY, visiableCircles, OBJC_ASSOCIATION_RETAIN);
}
- (NSMutableArray<NBCircle*>*)visiableCircles{
    NSMutableArray<NBCircle*>* visiableCircles = objc_getAssociatedObject(self, &NB_CIRCLE_OVERLAYS_KEY);
    if(visiableCircles == nil) {
        visiableCircles = [NSMutableArray arrayWithCapacity:0];
        [self setVisiableCircles:visiableCircles];
    }
    return visiableCircles;
}
- (void)setShouldCenterVisiableOverlays:(BOOL)shouldCenterVisiableOverlays{
    objc_setAssociatedObject(self, &NB_SHOULD_CENTER_VISIABLE_OVERLAYS_KEY, @(shouldCenterVisiableOverlays), OBJC_ASSOCIATION_ASSIGN);
}
- (BOOL)shouldCenterVisiableOverlays{
    NSNumber* shouldCenterVisiableOverlays = objc_getAssociatedObject(self, &NB_SHOULD_CENTER_VISIABLE_OVERLAYS_KEY);
    return shouldCenterVisiableOverlays.boolValue;
}

- (void)raiseExpectionWithOverlay:(id<MAOverlay>)overlay {
    NSString* reason = [NSString stringWithFormat:@"Missing cached overlay %@ , please add it to `MAMapView(CenterAroundVisiableOverlays) category`", [overlay description]];
    NSException* expection = [[NSException alloc]initWithName:@"CenterAroundVisiableOverlaysDomainError" reason: reason userInfo:NULL];
    [expection raise];
}

@end
