//
//  NBCommonUtility.m
//  nb_map
//
//  Created by Guo Longxiang, (Longxiang.Guo@partner.bmw.com) on 2019/11/1.
//

#import "NBCommonUtility.h"
#import <MAMapKit/MAGeometry.h>
@implementation NBCommonUtility

+ (BOOL)polylineHitTestWithCoordinate:(CLLocationCoordinate2D)coordinate
                              mapView:(MAMapView*)mapView
                       polylinePoints:(MAMapPoint*)polylinePoints
                           pointCount:(NSInteger)pointCount
                            lineWidth:(CGFloat)lineWidth {
    if(pointCount <= 1 || polylinePoints == NULL || !mapView) {
        return NO;
    }
    
    CGPoint cgp1 = [mapView convertCoordinate:coordinate toPointToView:mapView];
    CGPoint cgp2 = CGPointMake(cgp1.x + lineWidth * 0.5, cgp1.y);
    
    MAMapPoint tappedMapPoint = MAMapPointForCoordinate(coordinate);
    MAMapPoint mapPoint2 = MAMapPointForCoordinate([mapView convertPoint:cgp2 toCoordinateFromView:mapView]);
    
    CGFloat lengthSq = (tappedMapPoint.x - mapPoint2.x)*(tappedMapPoint.x - mapPoint2.x) + (tappedMapPoint.y - mapPoint2.y)*(tappedMapPoint.y - mapPoint2.y);
    
    
    BOOL hit = NO;
    for(int i = 0; i < pointCount - 1; ++i) {
        MAMapPoint begin = polylinePoints[i];
        MAMapPoint end = polylinePoints[i+1];
        
        CGFloat length2BeginSq = (tappedMapPoint.x - begin.x)*(tappedMapPoint.x - begin.x) + (tappedMapPoint.y - begin.y)*(tappedMapPoint.y - begin.y);
        if(length2BeginSq < lengthSq) {
            hit = YES;
            break;
        }
        
        CGFloat length2EndSq = (tappedMapPoint.x - end.x)*(tappedMapPoint.x - end.x) + (tappedMapPoint.y - end.y)*(tappedMapPoint.y - end.y);
        if(length2EndSq < lengthSq) {
            hit = YES;
            break;
        }
        
        MAMapPoint mappedPoint = [self getVerticalMappedPointOfPoint:tappedMapPoint toLineWithBegin:begin andEnd:end];
        CGFloat length2MappedPointSq = (tappedMapPoint.x - mappedPoint.x)*(tappedMapPoint.x - mappedPoint.x) + (tappedMapPoint.y - mappedPoint.y)*(tappedMapPoint.y - mappedPoint.y);
        if(length2MappedPointSq < lengthSq &&
           mappedPoint.y <= fmax(begin.y, end.y) && mappedPoint.y >= fmin(begin.y, end.y) &&
           mappedPoint.x <= fmax(begin.x, end.x) && mappedPoint.x >= fmin(begin.x, end.x)) {
            hit = YES;
            break;
        }
    }
    
    return hit;
}

+ (MAMapPoint)getVerticalMappedPointOfPoint:(MAMapPoint)pt toLineWithBegin:(MAMapPoint)begin andEnd:(MAMapPoint)end {
    MAMapPoint retVal;
    
    double dx = begin.x - end.x;
    double dy = begin.y - end.y;
    if(fabs(dx) < 0.00000001 && fabs(dy) < 0.00000001 ) {
        retVal = begin;
        return retVal;
    }
    
    double u = (pt.x - begin.x)*(begin.x - end.x) +
    (pt.y - begin.y)*(begin.y - end.y);
    u = u/((dx*dx)+(dy*dy));
    
    retVal.x = begin.x + u*dx;
    retVal.y = begin.y + u*dy;
    
    return retVal;
}


+ (MAMapRect)unionMapRect1:(MAMapRect)mapRect1 mapRect2:(MAMapRect)mapRect2
{
    CGRect rect1 = CGRectMake(mapRect1.origin.x, mapRect1.origin.y, mapRect1.size.width, mapRect1.size.height);
    CGRect rect2 = CGRectMake(mapRect2.origin.x, mapRect2.origin.y, mapRect2.size.width, mapRect2.size.height);
    
    CGRect unionRect = CGRectUnion(rect1, rect2);
    
    return MAMapRectMake(unionRect.origin.x, unionRect.origin.y, unionRect.size.width, unionRect.size.height);
}

+ (MAMapRect)mapRectUnion:(MAMapRect *)mapRects count:(NSUInteger)count
{
    if (mapRects == NULL || count == 0)
    {
        NSLog(@"%s: 无效的参数.", __func__);
        return MAMapRectZero;
    }
    
    MAMapRect unionMapRect = mapRects[0];
    
    for (int i = 1; i < count; i++)
    {
        unionMapRect = [self unionMapRect1:unionMapRect mapRect2:mapRects[i]];
    }
    
    return unionMapRect;
}

+ (MAMapRect)mapRectForOverlays:(NSArray<id<MAOverlay>> *)overlays
{
    if (overlays.count == 0)
    {
        NSLog(@"%s: 无效的参数.", __func__);
        return MAMapRectZero;
    }
    
    MAMapRect mapRect;
    
    MAMapRect *buffer = (MAMapRect*)malloc(overlays.count * sizeof(MAMapRect));
    
    [overlays enumerateObjectsUsingBlock:^(id<MAOverlay> obj, NSUInteger idx, BOOL *stop) {
        buffer[idx] = [obj boundingMapRect];
    }];
    
    mapRect = [self mapRectUnion:buffer count:overlays.count];
    
    (void)(free(buffer)), buffer = NULL;
    
    return mapRect;
}

+ (MAMapRect)minMapRectForMapPoints:(MAMapPoint *)mapPoints count:(NSUInteger)count
{
    if (mapPoints == NULL || count <= 1)
    {
        NSLog(@"%s: 无效的参数.", __func__);
        return MAMapRectZero;
    }
    
    CGFloat minX = mapPoints[0].x, minY = mapPoints[0].y;
    CGFloat maxX = minX, maxY = minY;
    
    /* Traverse and find the min, max. */
    for (int i = 1; i < count; i++)
    {
        MAMapPoint point = mapPoints[i];
        
        if (point.x < minX)
        {
            minX = point.x;
        }
        
        if (point.x > maxX)
        {
            maxX = point.x;
        }
        
        if (point.y < minY)
        {
            minY = point.y;
        }
        
        if (point.y > maxY)
        {
            maxY = point.y;
        }
    }
    
    /* Construct outside min rectangle. */
    return MAMapRectMake(minX, minY, fabs(maxX - minX), fabs(maxY - minY));
}

+ (MAMapRect)minMapRectForAnnotations:(NSArray *)annotations
{
    if (annotations.count <= 1)
    {
        NSLog(@"%s: 无效的参数.", __func__);
        return MAMapRectZero;
    }
    
    MAMapPoint *mapPoints = (MAMapPoint*)malloc(annotations.count * sizeof(MAMapPoint));
    
    [annotations enumerateObjectsUsingBlock:^(id<MAAnnotation> obj, NSUInteger idx, BOOL *stop) {
        mapPoints[idx] = MAMapPointForCoordinate([obj coordinate]);
    }];
    
    MAMapRect minMapRect = [self minMapRectForMapPoints:mapPoints count:annotations.count];
    
    (void)(free(mapPoints)), mapPoints = NULL;
    
    return minMapRect;
}

+ (double)distanceToPoint:(MAMapPoint)p fromLineSegmentBetween:(MAMapPoint)l1 and:(MAMapPoint)l2
{
    double A = p.x - l1.x;
    double B = p.y - l1.y;
    double C = l2.x - l1.x;
    double D = l2.y - l1.y;
    
    double dot = A * C + B * D;
    double len_sq = C * C + D * D;
    double param = dot / len_sq;
    
    double xx, yy;
    
    if (param < 0 || (l1.x == l2.x && l1.y == l2.y)) {
        xx = l1.x;
        yy = l1.y;
    }
    else if (param > 1) {
        xx = l2.x;
        yy = l2.y;
    }
    else {
        xx = l1.x + param * C;
        yy = l1.y + param * D;
    }
    
    return MAMetersBetweenMapPoints(p, MAMapPointMake(xx, yy));
}

@end

