//
//  NBJsonConversions.m
//  nb_map
//
//  Created by Guo Longxiang, (Longxiang.Guo@partner.bmw.com) on 2019/10/26.
//

#import "NBJsonConversions.h"

@implementation NBJsonConversions

+ (bool)toBool:(NSNumber*)data {
    return data.boolValue;
}

+ (int)toInt:(NSNumber*)data {
    return data.intValue;
}

+ (double)toDouble:(NSNumber*)data {
    return data.doubleValue;
}

+ (float)toFloat:(NSNumber*)data {
    return data.floatValue;
}

+ (CLLocationCoordinate2D)toLocation:(NSArray*)data {
    return CLLocationCoordinate2DMake([NBJsonConversions toDouble:data[0]],
                                      [NBJsonConversions toDouble:data[1]]);
}

+ (CGPoint)toPoint:(NSArray*)data {
    return CGPointMake([NBJsonConversions toDouble:data[0]],
                       [NBJsonConversions toDouble:data[1]]);
}

+ (NSArray*)positionToJson:(CLLocationCoordinate2D)position {
    return @[ @(position.latitude), @(position.longitude) ];
}

+ (UIColor*)toColor:(NSNumber*)numberColor {
    unsigned long value = [numberColor unsignedLongValue];
    return [UIColor colorWithRed:((float)((value & 0xFF0000) >> 16)) / 255.0
                           green:((float)((value & 0xFF00) >> 8)) / 255.0
                            blue:((float)(value & 0xFF)) / 255.0
                           alpha:((float)((value & 0xFF000000) >> 24)) / 255.0];
}

+ (NSArray<CLLocation*>*)toPoints:(NSArray*)data {
    NSMutableArray* points = [[NSMutableArray alloc] init];
    for (unsigned i = 0; i < [data count]; i++) {
        NSNumber* latitude = data[i][0];
        NSNumber* longitude = data[i][1];
        CLLocation* point =
        [[CLLocation alloc] initWithLatitude:[NBJsonConversions toDouble:latitude]
                                   longitude:[NBJsonConversions toDouble:longitude]];
        [points addObject:point];
    }
    
    return points;
}

@end
