//
//  NBMapCircleController.h
//  nb_map
//
//  Created by Guo Longxiang, (Longxiang.Guo@partner.bmw.com) on 2019/10/29.
//

#import <Foundation/Foundation.h>
#import <MAMapKit/MAMapKit.h>
#import <Flutter/Flutter.h>

// 定义视图接受Flutter API接口
@protocol NBMapCircleOptionsSink
- (void)setConsumeTapEvents:(BOOL)consume;
- (void)setVisible:(BOOL)visible;
- (void)setStrokeColor:(UIColor*)color;
- (void)setStrokeWidth:(CGFloat)width;
- (void)setFillColor:(UIColor*)color;
- (void)setCenter:(CLLocationCoordinate2D)center;
- (void)setRadius:(CLLocationDistance)radius;
- (void)setZIndex:(int)zIndex;
- (void)setNeedsUpdate;
@end

///管理视图控的UI设定和Flutter端的事件交互
@interface NBMapCircleController : NSObject<NBMapCircleOptionsSink>
@property(atomic, readonly) NSString* circleId;
- (instancetype)initCircleWithPosition:(CLLocationCoordinate2D)position
                                radius:(CLLocationDistance)radius
                              circleId:(NSString*)circleId
                               mapView:(MAMapView*)mapView;
- (void)addCircle;
- (void)removeCircle;

@end

///管理视图控制器,对父视图的事件进行统一的分发,子视图的事件进行统一管理
@interface NBMapCirclesController : NSObject
- (instancetype)init:(FlutterMethodChannel*)methodChannel
             mapView:(MAMapView*)mapView
           registrar:(NSObject<FlutterPluginRegistrar>*)registrar;
- (void)addCircles:(NSArray*)circlesToAdd;
- (void)changeCircles:(NSArray*)circlesToChange;
- (void)removeCircleIds:(NSArray*)circleIdsToRemove;
- (void)onCircleTap:(NSString*)circleId;
- (bool)hasCircleWithId:(NSString*)circleId;
@end

///适配Google API,衍生的子类,负责更新对应的Render绘制属性以及视图手势事件。
@interface NBCircle : MACircle
@property (nonatomic, assign) BOOL tappable;
@property (nonatomic, assign, getter=isVisiable) BOOL visible;
@property (nonatomic, assign) CGFloat lineWidth;
@property (nonatomic, retain) UIColor *fillColor;
@property (nonatomic, retain) UIColor *strokeColor;
@property (nonatomic, assign) MAOverlayLevel level;
@end
