//
//  NBMapMarkerController.h
//  nb_map
//
//  Created by Guo Longxiang, (Longxiang.Guo@partner.bmw.com) on 2019/10/29.
//

#import <Foundation/Foundation.h>
#import <MAMapKit/MAMapKit.h>
#import <Flutter/Flutter.h>

@protocol NBMapMarkerOptionsSink
- (void)setAlpha:(float)alpha;
///Annotation视图的锚点
- (void)setAnchor:(CGPoint)anchor;
///Annotation弹框的锚点
- (void)setInfoWindowAnchor:(CGPoint)anchor;
///Annotation弹框的title和subTitle
- (void)setInfoWindowTitle:(NSString*)title snippet:(NSString*)snippet;
- (void)setConsumeTapEvents:(BOOL)consume;
- (void)setDraggable:(BOOL)draggable;
- (void)setIcon:(UIImage*)icon;
- (void)setPosition:(CLLocationCoordinate2D)position;
- (void)setRotation:(CLLocationDegrees)rotation;
- (void)setVisible:(BOOL)visible;
- (void)setShowCallOut:(BOOL)showCallOut;
- (void)setZIndex:(int)zIndex;
- (void)setNeedsUpdate;
@end

///管理视图控的UI设定和Flutter端的事件交互
@interface NBMapMarkerController : NSObject <NBMapMarkerOptionsSink>
@property(atomic, readonly) NSString* markerId;
@property(nonatomic, assign) BOOL shouldConsumeTapEvents;
- (instancetype)initMarkerWithPosition:(CLLocationCoordinate2D)position
                              markerId:(NSString*)markerId
                               mapView:(MAMapView*)mapView;
- (void)addMarker;
- (void)removeMarker;
@end

///管理视图控制器,对父视图的事件进行统一的分发,子视图的事件进行统一管理
@interface NBMapMarkersController : NSObject
- (instancetype)init:(FlutterMethodChannel*)methodChannel
             mapView:(MAMapView*)mapView
           registrar:(NSObject<FlutterPluginRegistrar>*)registrar;
- (void)addMarkers:(NSArray*)markersToAdd;
- (void)changeMarkers:(NSArray*)markersToChange;
- (void)removeMarkerIds:(NSArray*)markerIdsToRemove;
- (BOOL)onMarkerTap:(NSString*)markerId;
- (void)onMarkerDragEnd:(NSString*)markerId coordinate:(CLLocationCoordinate2D)coordinate;
- (void)onInfoWindowTap:(NSString*)markerId;
@end

///适配Google API,衍生的子类,UI部分可以修改的属性过多,直接定制AnnotationView。
@interface NBMarker : MAPointAnnotation

@property (nonatomic, assign) BOOL tappable;
@property (nonatomic, assign, getter=isVisiable) BOOL visiable;
@property (nonatomic, assign) BOOL draggable;
@property (nonatomic, assign) CGFloat alpha;
@property (nonatomic, assign) CGPoint anchor;
@property (nonatomic, assign) CGPoint windowInfoAnchor;
@property (nonatomic, assign) CLLocationDegrees rotation;
@property (nonatomic, assign) MAOverlayLevel level;
@property (nonatomic, assign) BOOL canShowCallOut;
@property (nonatomic, assign) BOOL canShowAccessoryView;
@property (nonatomic, strong) UIImage* icon;

@end
