//
//  NBMapPloygonController.h
//  nb_map
//
//  Created by Guo Longxiang, (Longxiang.Guo@partner.bmw.com) on 2019/10/29.
//

#import <Foundation/Foundation.h>
#import <MAMapKit/MAMapKit.h>
#import <Flutter/Flutter.h>

// 定义视图接受Flutter API接口
@protocol NBMapPolygonOptionsSink
- (void)setConsumeTapEvents:(BOOL)consume;
- (void)setVisible:(BOOL)visible;
- (void)setFillColor:(UIColor*)color;
- (void)setStrokeColor:(UIColor*)color;
- (void)setStrokeWidth:(CGFloat)width;
- (void)setPoints:(NSArray<CLLocation*>*)points;
- (void)setZIndex:(int)zIndex;
- (void)setNeedsUpdate;
@end

///管理视图控的UI设定和Flutter端的事件交互
@interface NBMapPloygonController : NSObject<NBMapPolygonOptionsSink>

@property(atomic, readonly) NSString* polygonId;
- (instancetype)initPolygonWithCoord:(CLLocationCoordinate2D*)coords
                           polygonId:(NSString*)polygonId
                             mapView:(MAMapView*)mapView;
- (void)addPolygon;
- (void)removePolygon;

@end

///管理视图控制器,对父视图的事件进行统一的分发,子视图的事件进行统一管理
@interface NBMapPolygonsController : NSObject
- (instancetype)init:(FlutterMethodChannel*)methodChannel
             mapView:(MAMapView*)mapView
           registrar:(NSObject<FlutterPluginRegistrar>*)registrar;
- (void)addPolygons:(NSArray*)polygonsToAdd;
- (void)changePolygons:(NSArray*)polygonsToChange;
- (void)removePolygonIds:(NSArray*)polygonIdsToRemove;
- (void)onPolygonTap:(NSString*)polygonId;
- (bool)hasPolygonWithId:(NSString*)polygonId;
@end

///适配Google API,衍生的子类,负责更新对应的Render绘制属性以及视图手势事件。
@interface NBPolygon : MAPolygon
@property (nonatomic, assign) BOOL tappable;
@property (nonatomic, assign, getter=isVisiable) BOOL visible;
@property (nonatomic, assign) CGFloat lineWidth;
@property (nonatomic, retain) UIColor *fillColor;
@property (nonatomic, retain) UIColor *strokeColor;
@property (nonatomic, assign) MAOverlayLevel level;

@end
