//
//  NBMapPolylineController.h
//  nb_map
//
//  Created by Guo Longxiang, (Longxiang.Guo@partner.bmw.com) on 2019/10/29.
//

#import <Foundation/Foundation.h>
#import <MAMapKit/MAMapKit.h>
#import <Flutter/Flutter.h>

// 定义视图接受Flutter API接口
@protocol NBMapPolylineOptionsSink
- (void)setConsumeTapEvents:(BOOL)consume;
- (void)setVisible:(BOOL)visible;
- (void)setColor:(UIColor*)color;
- (void)setStrokeWidth:(CGFloat)width;
- (void)setPoints:(NSArray<CLLocation*>*)points;
- (void)setZIndex:(int)zIndex;
- (void)setNeedsUpdate;
@end

///管理视图控的UI设定和Flutter端的事件交互
@interface NBMapPolylineController : NSObject<NBMapPolylineOptionsSink>
@property (atomic, readonly) NSString* polylineId;
- (instancetype)initPolylineWithCoords:(CLLocationCoordinate2D*)coords
                                 count:(NSUInteger)count
                            polylineId:(NSString*)polylineId
                               mapView:(MAMapView*)mapView;
- (void)addPolyine;
- (void)removePolyline;
@end

///管理视图控制器,对父视图的事件进行统一的分发,子视图的事件进行统一管理
@interface NBMapPolylinesController : NSObject
- (instancetype)init:(FlutterMethodChannel*)methodChannel
             mapView:(MAMapView*)mapView
           registrar:(NSObject<FlutterPluginRegistrar>*)registrar;
- (void)addPolylines:(NSArray*)polylinesToAdd;
- (void)changePolylines:(NSArray*)polylinesToChange;
- (void)removePolylineIds:(NSArray*)polylineIdsToRemove;
- (void)onPolylineTap:(NSString*)polylineId;
- (bool)hasPolylineWithId:(NSString*)polylineId;
@end

///适配Google API,衍生的子类,负责更新对应的Render绘制属性以及视图手势事件。
@interface NBPolyline : MAPolyline
@property (nonatomic, assign) BOOL tappable;
@property (nonatomic, assign, getter=isVisiable) BOOL visible;
@property (nonatomic, assign) CGFloat lineWidth;
@property (nonatomic, retain) UIColor *fillColor;
@property (nonatomic, retain) UIColor *strokeColor;
@property (nonatomic, assign) MAOverlayLevel level;
@end
