//
//  NBMarkerViewsFactory.m
//  nb_map
//
//  Created by Guo Longxiang, (Longxiang.Guo@partner.bmw.com) on 2019/11/6.
//

#import "NBMarkerViewsFactory.h"



@implementation NBMarkerViewsFactory
+ (MAAnnotationView*)createUserAnnotationViewWith:(MAUserLocation*)userLocation mapView:(MAMapView*)mapView {
    return nil;
}
+ (MAAnnotationView*)createAnnotationViewWith:(NBMarker*)marker mapView:(MAMapView*)mapView {
    MAAnnotationView* annotationView = (MAAnnotationView*)[mapView dequeueReusableAnnotationViewWithIdentifier:@"NBMarkerView"];
    if (annotationView == nil) {
        annotationView = [[MAAnnotationView alloc]initWithAnnotation:marker reuseIdentifier:@"NBMarkerView"];
    }
    annotationView.image = [NBMarkerViewsFactory createMarkerViewWith:marker];
    annotationView.userInteractionEnabled = marker.tappable;
    annotationView.draggable = marker.draggable;
    annotationView.hidden = !marker.isVisiable;
    annotationView.alpha = marker.alpha;
    annotationView.layer.anchorPoint = marker.anchor;
    annotationView.zIndex = marker.level;
    annotationView.canShowCallout = marker.canShowCallOut;
    if ([marker canShowCallOut]) {
        annotationView.customCalloutView = [NBMarkerViewsFactory createCallOutViewWith:marker];
    }
    if ([marker canShowAccessoryView]){
        annotationView.leftCalloutAccessoryView = [NBMarkerViewsFactory createAccessoryViewWith:marker isLeft:true];
        annotationView.rightCalloutAccessoryView = [NBMarkerViewsFactory createAccessoryViewWith:marker isLeft:false];
    }
    return annotationView;
}
+ (UIImage*)createMarkerViewWith:(NBMarker*)marker {
    return marker.icon;
}

+ (MACustomCalloutView*)createCallOutViewWith:(NBMarker*)marker {
    MACustomCalloutView* callOutView = [[MACustomCalloutView alloc]initWithFrame:CGRectMake(0.0, 0.0, 50.0, 20.0)];
    callOutView.backgroundColor = UIColor.grayColor;
    return callOutView;
}

+ (UIView*)createAccessoryViewWith:(NBMarker*)marker isLeft:(BOOL)isLeft {
    UIView* accessoryView = [[UIView alloc]initWithFrame:CGRectMake(0.0, 0.0, 50.0, 20.0)];
    return accessoryView;
}
@end
